DynamicArray

Struct DynamicArray 

pub struct DynamicArray<Length, Collection> { /* private fields */ }
Dynamically sized array.

Used to serialize and deserialize arbitrary collections using specific length fields.

Works on any type that implements IntoIterator<Item = Serialize> which includes Vec<T>, BTreeMap<K, V>, and Option<T>.

The length can be LengthZero, LengthU8, LengthU16 or LengthU32 for 0, 8, 16, and 32 bit length fields, respectively.

Examples

use rsomeip_bytes::{Deserialize as _, DynamicArray, LengthU32, Serialize as _};

// Works on any value that implements `IntoIterator`.
let value = vec![1, 2, 3, 4, 5];

// Specify the size of the length field in the generic parameters. It will be included when
// serializing the value.
let array = DynamicArray::<LengthU32, _>::from(&value);

// The array also implements the `Serialize` trait.
let mut bytes = array.to_bytes()?;

// And `Deserialize`, as well.
let output = DynamicArray::<LengthU32, Vec<i32>>::deserialize(&mut bytes)?;
assert_eq!(value, output);